import { Hono } from 'hono';
import { handleTusRequest, cleanupExpiredUploads, getStorageInfo } from './tus';
import {
	getResourceByFileId,
	getAllResources,
	deleteResource,
	updateResource,
	getResourcesByStorageType,
	getResourcesByStatus,
	getUploadingResources,
	getResourceStats,
	migrateResourcesStorageType,
} from './upload.index';
import { StorageManager, StorageType, type StorageConfig } from './storage.adapter';
import { prisma } from '@repo/db';

const uploadRest = new Hono();

// 获取文件资源信息
uploadRest.get('/resource/:fileId', async (c) => {
	const fileId = c.req.param('fileId');
	const result = await getResourceByFileId(fileId);
	return c.json(result);
});

// 获取所有资源
uploadRest.get('/resources', async (c) => {
	const resources = await getAllResources();
	return c.json(resources);
});

// 根据存储类型获取资源
uploadRest.get('/resources/storage/:storageType', async (c) => {
	const storageType = c.req.param('storageType') as StorageType;
	const resources = await getResourcesByStorageType(storageType);
	return c.json(resources);
});

// 根据状态获取资源
uploadRest.get('/resources/status/:status', async (c) => {
	const status = c.req.param('status');
	const resources = await getResourcesByStatus(status);
	return c.json(resources);
});

// 获取正在上传的资源
uploadRest.get('/resources/uploading', async (c) => {
	const resources = await getUploadingResources();
	return c.json(resources);
});

// 获取资源统计信息
uploadRest.get('/stats', async (c) => {
	const stats = await getResourceStats();
	return c.json(stats);
});

// 删除资源
uploadRest.delete('/resource/:id', async (c) => {
	const id = c.req.param('id');
	const result = await deleteResource(id);
	return c.json(result);
});

// 更新资源
uploadRest.patch('/resource/:id', async (c) => {
	const id = c.req.param('id');
	const data = await c.req.json();
	const result = await updateResource(id, data);
	return c.json(result);
});

// 迁移资源存储类型（批量更新数据库中的存储类型标记）
uploadRest.post('/migrate-storage', async (c) => {
	try {
		const { from, to } = await c.req.json();
		const result = await migrateResourcesStorageType(from as StorageType, to as StorageType);
		return c.json({
			success: true,
			message: `Migrated ${result.count} resources from ${from} to ${to}`,
			count: result.count,
		});
	} catch (error) {
		console.error('Failed to migrate storage type:', error);
		return c.json(
			{
				success: false,
				error: error instanceof Error ? error.message : 'Unknown error',
			},
			400,
		);
	}
});

// 清理过期上传
uploadRest.post('/cleanup', async (c) => {
	const result = await cleanupExpiredUploads();
	return c.json(result);
});

// 手动清理指定状态的资源
uploadRest.post('/cleanup/by-status', async (c) => {
	try {
		const { status, olderThanDays } = await c.req.json();
		const cutoffDate = new Date();
		cutoffDate.setDate(cutoffDate.getDate() - (olderThanDays || 30));

		const deletedResources = await prisma.resource.deleteMany({
			where: {
				status,
				createdAt: {
					lt: cutoffDate,
				},
			},
		});

		return c.json({
			success: true,
			message: `Deleted ${deletedResources.count} resources with status ${status}`,
			count: deletedResources.count,
		});
	} catch (error) {
		console.error('Failed to cleanup by status:', error);
		return c.json(
			{
				success: false,
				error: error instanceof Error ? error.message : 'Unknown error',
			},
			400,
		);
	}
});

// 获取存储信息
uploadRest.get('/storage/info', async (c) => {
	const storageInfo = getStorageInfo();
	return c.json(storageInfo);
});

// 切换存储类型（需要重启应用）
uploadRest.post('/storage/switch', async (c) => {
	try {
		const newConfig = (await c.req.json()) as StorageConfig;
		const storageManager = StorageManager.getInstance();
		await storageManager.switchStorage(newConfig);

		return c.json({
			success: true,
			message: 'Storage configuration updated. Please restart the application for changes to take effect.',
			newType: newConfig.type,
		});
	} catch (error) {
		console.error('Failed to switch storage:', error);
		return c.json(
			{
				success: false,
				error: error instanceof Error ? error.message : 'Unknown error',
			},
			400,
		);
	}
});

// 验证存储配置
uploadRest.post('/storage/validate', async (c) => {
	try {
		const config = (await c.req.json()) as StorageConfig;
		const { validateStorageConfig } = await import('./storage.adapter');
		const errors = validateStorageConfig(config);

		if (errors.length > 0) {
			return c.json({ valid: false, errors }, 400);
		}

		return c.json({ valid: true, message: 'Storage configuration is valid' });
	} catch (error) {
		return c.json(
			{
				valid: false,
				errors: [error instanceof Error ? error.message : 'Invalid JSON'],
			},
			400,
		);
	}
});

// TUS 协议处理 - 使用通用处理器
uploadRest.all('/*', async (c) => {
	try {
		await handleTusRequest(c.req.raw, c.res);
		return new Response(null);
	} catch (error) {
		console.error('TUS request error:', error);
		return c.json({ error: 'Upload request failed' }, 500);
	}
});

export default uploadRest;
